package com.webchat.aigc.llm;


import com.webchat.common.constants.ConnectConstants;
import com.webchat.common.enums.AiFunctionEnum;
import com.webchat.common.helper.SseEmitterHelper;
import com.webchat.common.util.JsonUtil;
import com.webchat.domain.vo.llm.FunctionCallResponse;
import com.webchat.domain.vo.llm.image.ImageGenerateResponse;
import com.webchat.domain.vo.llm.image.RequestGenerateBaseParam;
import com.webchat.domain.vo.llm.image.RequestGenerateParam;
import com.webchat.domain.vo.request.mess.ChatMessageRequestVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;

import java.util.List;

@Slf4j
@Service
public class AiAgentFlowService {

    @Autowired
    private GPTChatService gptChatService;
    @Autowired
    private AiFunctionCallService aiFunctionCallService;
    @Autowired
    private LibLibGenerateImageService libLibGenerateImageService;

    /**
     * 旗舰版文生图模版id
     */
    private static final String TEMPLATE_UUID = "5d7e67009b344550bc1aa6ccbfa1d7f4";

    private String getSSEBizCode() {
        return ConnectConstants.ConnectBiz.getBizCode(ConnectConstants.ClientEnum.PC,
                                                      ConnectConstants.AppEnum.WEB,
                                                      ConnectConstants.BizEnum.CHAT);
    }

    /**
     * 机器人问答Agent服务
     * 1、意图识别
     * 2、文生文（kimi/deepseek） / 文生图（LibLib模型）
     *
     * @param message
     */
    public void doAgent(String message) {
        ChatMessageRequestVO chatMessage = JsonUtil.fromJson(message, ChatMessageRequestVO.class);
        String senderId = chatMessage.getSenderId();
        /**
         * 1. 意图识别
         */
        SseEmitterHelper.send(this.getSSEBizCode(), senderId, "意图识别中...");
        FunctionCallResponse functionCallResponse;
        try {
            functionCallResponse =
                    aiFunctionCallService.getFunction(chatMessage.getMessage());
            log.info("意图识别结果 =====> input: {}, response:{}",
                    chatMessage.getMessage(), JsonUtil.toJsonString(functionCallResponse));
            String functionInfo = "意图识别："+ AiFunctionEnum.getFunctionName(functionCallResponse.getFunction());
            SseEmitterHelper.send(this.getSSEBizCode(), senderId, functionInfo);
        } catch (Exception e) {
            log.error("意图识别异常 =====> input: {}", chatMessage.getMessage(), e);
            SseEmitterHelper.send(this.getSSEBizCode(), senderId, "意图识别失败，稍后重试～");
            return;
        }
        /**
         * 2. 根据意图执行对应工作流
         */
        SseEmitter sseEmitter = SseEmitterHelper.get(this.getSSEBizCode(), senderId);
        if (AiFunctionEnum.CHAT.name().equals(functionCallResponse.getFunction())) {
            // 机器人对话
            gptChatService.chat(sseEmitter, chatMessage);
        } else if (AiFunctionEnum.IMAGE.name().equals(functionCallResponse.getFunction())) {
            // 机器人对话
            RequestGenerateBaseParam baseParam = RequestGenerateBaseParam.of(chatMessage.getMessage());
            RequestGenerateParam param = RequestGenerateParam.of(TEMPLATE_UUID, baseParam);
            try {
                ImageGenerateResponse response = libLibGenerateImageService.generate(senderId, param);
                List<String> images = response.getImages();
                if (CollectionUtils.isEmpty(images)) {
                    SseEmitterHelper.send(this.getSSEBizCode(), senderId, "图片生成失败～");
                    return;
                }
                String imageMd = " ![%s](%s \"%s\")";
                StringBuilder stringBuilder = new StringBuilder();
                for (int i = 0; i < images.size(); i++) {
                    String imageName = "图片" + (i + 1);
                    stringBuilder.append(String.format(imageMd, imageName, images.get(i), imageName));
                    if (i == images.size() - 1) {
                        stringBuilder.append("\n");
                    }
                }
                SseEmitterHelper.send(this.getSSEBizCode(), senderId, stringBuilder.toString());
            } catch (Exception e) {
                log.error("文生图服务异常 =====> param: {}", JsonUtil.toJsonString(param), e);
                SseEmitterHelper.send(this.getSSEBizCode(), senderId, "意图识别失败，稍后重试～");
            }
        }
    }
}
