package com.webchat.connect.messagequeue.consumer.redis;

import com.webchat.connect.messagequeue.consumer.service.ChatNotifyConsumeService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.connection.MessageListener;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import java.util.concurrent.ArrayBlockingQueue;

@Slf4j
@Component
public class ChatNotifyRedisQueueListener implements MessageListener {

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private ChatNotifyConsumeService chatNotifyConsumeService;

    private static final int TASK_QUEUE_SIZE = 2000;

    /**
     * 阻塞队列入队、消费异常，重试次数
     */
    private static final int RETRY_TIMES = 3;

    private static ArrayBlockingQueue<String> taskQueue = new ArrayBlockingQueue<>(TASK_QUEUE_SIZE);

    @Override
    public void onMessage(Message message, byte[] pattern) {
        String channel = (String) redisTemplate.getStringSerializer().deserialize(message.getChannel());
        String messageStr = (String) redisTemplate.getValueSerializer().deserialize(message.getBody());
        log.info("ChatMessageNotifyQueueListener.onMessage =====> channel:{} messageStr:{}", channel, messageStr);
        // 加入任务队列（遇到通知大流量场景，缓解consumer压力）
        this.putTaskQueue(messageStr);
    }

    public void consumeTask() {
        while (true) {
            String message = "";
            try {
                message = taskQueue.take();
                if (StringUtils.isNotBlank(message)) {
                    chatNotifyConsumeService.doNotify(message);
                }
            } catch (InterruptedException e) {
                log.error("通知消息队列消费异常！message:{}", message, e);
                this.putTaskQueue(message);
            } catch (Exception e) {
                log.error("通知消息队列消费异常！message:{}", message, e);
            }
        }
    }

    /**
     * 重新加入队列
     * @param message
     */
    private void putTaskQueue(String message) {
        int retryTimes = 0;
        while (retryTimes <= RETRY_TIMES) {
            try {
                taskQueue.put(message);
                return;
            } catch (InterruptedException e) {
                log.error("通知消息加入队列异常！message:{}", message, e);
                retryTimes ++;
            }
        }
    }
}
